#!/bin/sh

# Preprocessor for 'less'. Used when this environment variable is set:
# LESSOPEN="|lesspipe.sh %s"

trap 'exit 0' PIPE

guesscompress() {
	case "$1" in
		*.gz)   echo "gunzip -c" ;;
		*.bz2)  echo "bunzip2 -c" ;;
		*.Z)    echo "compress -d" ;;
		*)      echo "cat" ;;
	esac
}

lesspipe_file() {
	local out=`file -L -- "$1"`
	case ${out} in
		*" tar archive"*)   		lesspipe "$1" ".tar" ;;
		*" troff "*)        		lesspipe "$1" ".man" ;;
		*" shared object"*) 		lesspipe "$1" ".so" ;;
		*" cpio archive"*)  		lesspipe "$1" ".cpio" ;;
		*" current ar archive"*)	objdump -t -D -- "$1" ;;
		*" Zip archive data"*)		zipinfo -- "$1" ;;
		*" ELF "*)          		readelf -a -- "$1" ;;
		*" compiled Java class data"*)	jcf-dump -v -c -- "$1" ;;
		*": data")          		hexdump -C -- "$1" ;;
		*)                  		return 1 ;;
	esac
	return 0
}

lesspipe() {
	local match=$2
	[[ -z ${match} ]] && match=$1
	
	local DECOMPRESSOR=$(guesscompress "$match")
	
	case "$match" in
	
	### Manuals ###
	*.[0-9n]|*.man|\
	*.[0-9n].bz2|*.man.bz2|\
	*.[0-9n].gz|*.man.gz|\
	*.[0-9][a-z].gz|*.[0-9][a-z].gz)
		local out=$(${DECOMPRESSOR} -- "$1" | file -)
		case ${out} in
			*troff*)
				# Need to make sure we pass path to man or it will try
				# to locate "$1" in the man search paths
				if [[ $1 == /* ]] ; then
					man -- "$1"
				else
					man -- "./$1"
				fi
				;;
			*text*)
				${DECOMPRESSOR} -- "$1"
				;;
			*)
				# We could have matched a library, so let `file` 
				# figure out what the hell this thing is
				lesspipe_file "$1"
				;;
		esac
		;;
	*.conf|*.txt|*.text|*.info|*.log) ;; # force less to work on these directly
	
	### URLs ###
	ftp://*|http://*|*.htm|*.html)
		for b in links2 links lynx ; do
			${b} -dump "$1" && exit 0
		done
		html2text -style pretty "$1"
		;;

	### Tar files ###
	*.tar)                  tar tvvf "$1" ;;
	*.tar.bz2|*.tbz2|*.tbz) tar tjvvf "$1" ;;
	*.tar.gz|*.tgz|*.tar.z) tar tzvvf "$1" ;;

	### Misc archives ###
	*.bz2)        bzip2 -dc -- "$1" ;;
	*.gz|*.z)     gzip -dc -- "$1"  ;;
	*.cpio) cpio -itv < "$1" ;;
	*.so)         readelf -h -d -s -- "$1" ;;
	
	### Everything else ###
	*)
		# Sanity check
		[[ ${recur} == 2 ]] && exit 0

		# Maybe we didn't match due to case issues ...
		if [[ ${recur} == 0 ]] ; then
			recur=1
			lesspipe "$1" "$(echo $1 | tr '[:upper:]' '[:lower:]')"
		else
			recur=2
			lesspipe_file "$1"
		fi

		exit 0
		;;
	esac
}

if [[ -z $1 ]] ; then
	echo "Usage: lesspipe.sh <file>"
elif [[ $1 == "-V" ]] ; then
	printf "%s\n\n" "lesspipe.sh, v1.1.3"
	less -V
elif [[ $1 == "-h" || $1 == "--help" ]] ; then
	printf "%s\n%s\n%s\n" "lesspipe.sh: preproccess files before sending them to less" \
		"Usage: lesspipe.sh <file>" \
		"Run 'less --help' or 'man less' for more info"
elif [[ -d $1 ]] ; then
	ls -alF -- "$1"
else
	recur=0
	lesspipe "$1" 2> /dev/null
fi
